#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>

#include "uart.h"
#include "iinchip/types.h"
#include "iinchip/w5100.h"
#include "iinchip/socket.h"

#if (__DEF_IINCHIP_BUS__ == __DEF_IINCHIP_SPI_MODE__)
#include "iinchip/spi.h"
#endif  

void InitMCU(void);
void InitNET(void);
void DisplayConfig(void);

#define MY_NET_MAC			{0x00, 0x08, 0xdc, 0x00, 0x00, 0x00}
#define MY_NET_GWIP			{192,  168, 1,  94}	// MY Gateway IP
#define MY_DESTINATIONIP	{192,  168, 1,  91}	// MY Destination
#define MY_SOURCEIP			{192,  168, 1,  92}	// MY Source IP
#define MY_SUBNET			{255, 255, 255,  0} // MY Subnet Mask

#define MAX_BUF_SIZE			1024

//-----------------------------------------------------------------------------
void InitNET(void)
{
	unsigned char mac[6] = MY_NET_MAC;
	unsigned char sm[4]	= MY_SUBNET;
	unsigned char gwip[4] = MY_NET_GWIP;
	unsigned char sip[4] = MY_SOURCEIP;

	//W5100 Chip Init
	iinchip_init();

	//Set MAC Address
	setSHAR(mac);

	//Set Gateway
	setGAR(gwip);

	//Set Subnet Mask
	setSUBR(sm);

	//Set My IP
	setSIPR(sip);
	
#ifdef __DEF_IINCHIP_INT__
	setIMR(0xEF);
#endif
	
	sysinit(0x55, 0x55);
}
//-----------------------------------------------------------------------------


void test(unsigned char *pBuff, unsigned int Len)
{
	int i =0;
	for(i=0;i<Len;i++)
	{
		printf("%02x ", pBuff[i]);
	}
}

//-----------------------------------------------------------------------------
/**
 * UDP loopback program.
 */ 
void     loopback_udp(SOCKET s, uint16 port)
{
	unsigned int len;
	unsigned char buf[MAX_BUF_SIZE];
	unsigned char destip[4];
	unsigned int destport;

	switch(getSn_SR(s))
	{
														 // -------------------------------
	  case SOCK_UDP:                                     // 
		 if((len=getSn_RX_RSR(s)) > 0)                   // check the size of received data
		 {
			len = recvfrom(s,buf,len,destip,&destport);  // receive data from a destination
			
			if(len !=sendto(s,buf,len,destip,destport))  // send the data to the destination
			{
			   printf("%d : Sendto Fail.len=%ld,",s,len);
			   printf("%d.%d.%d.%d(%d)\r\n",destip[0],destip[1],destip[2],destip[3],destport);
			}
		 }
		 break;
														 // -----------------
	  case SOCK_CLOSED:                                  // CLOSED
		 close(s);                                       // close the SOCKET
		 socket(s,Sn_MR_UDP,port,0);                  // open the SOCKET with UDP mode
		 break;
	  default:
		 break;
	}
}


//-----------------------------------------------------------------------------
// Main function
//-----------------------------------------------------------------------------
void main(void)
{
	InitMCU();
	initUART(); 
	InitNET(); 

	printf("W5100 Simple UDP Loopback Test Program.\r\n");
	DisplayConfig();
	
	while(1)
	{
		loopback_udp(0, 5000);
		loopback_udp(1, 5001);		
		loopback_udp(2, 5002);
		loopback_udp(3, 5003);		
	}
}
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Display net info
void DisplayConfig(void)
{
	unsigned char temp[6];
	unsigned char i = 0;
	
	printf("\r\n================================================\r\n");
	printf("       Net Config Information\r\n");
	printf("================================================\r\n");
	
	printf("MAC ADDRESS      : ");
	getSHAR(temp);
	for(i=0; i<6;i++)printf("%02X:",temp[i]);
	

	printf("\r\nSUBNET MASK      : ");
	getSUBR(temp);
	for(i=0; i < 4; i++)
	{
		printf("%d.", temp[i]);
	}


	printf("\r\nG/W IP ADDRESS   : ");
	getGAR(temp);
	for(i=0; i < 4; i++)
	{
		printf("%d.", temp[i]);
	}

	printf("\r\nLOCAL IP ADDRESS : ");
	getSIPR(temp);
	for(i=0; i < 4; i++)
	{
		printf("%d.", temp[i]);
	}

	printf("\r\n================================================\r\n");	
}


//-----------------------------------------------------------------------------
#define	ATMEGA128_0WAIT		0
#define	ATMEGA128_1WAIT		1
#define	ATMEGA128_2WAIT		2
#define	ATMEGA128_3WAIT		3
#define ATMEGA128_NUM_WAIT	ATMEGA128_3WAIT

//MCU Initialize
void InitMCU(void) 
{
	cli();
		
#ifndef __DEF_IINCHIP_INT__	
	EICRA=0x00;
	EICRB=0x00;
	EIMSK=0x00;
	EIFR=0x00;
#else
	EICRA = 0x00;			// External Interrupt Control Register A clear
	EICRB = 0x02;			// External Interrupt Control Register B clear // edge 
	EIMSK = (1 << INT4);		// External Interrupt Mask Register : 0x10
	EIFR = 0xFF;			// External Interrupt Flag Register all clear
	DDRE &= ~(1 << INT4);		// Set PE Direction 
	PORTE |= (1 << INT4);		// Set PE Default value
	
	sei();				// enable interrupts
#endif
 

#if (ATMEGA128_NUM_WAIT == ATMEGA128_0WAIT)
	MCUCR = 0x80;		
	XMCRA=0x40;
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_1WAIT)
	MCUCR = 0xc0;		// MCU control regiseter : enable external ram
	XMCRA=0x40;		// External Memory Control Register A : 
						// Low sector   : 0x1100 ~ 0x7FFF
						// Upper sector : 0x8000 ~ 0xFFFF
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_2WAIT )
	MCUCR = 0x80;
	XMCRA=0x42;
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_3WAIT)
	MCUCR = 0xc0;
	XMCRA=0x42;
#else
#error "unknown atmega128 number wait type"
#endif	


	#if (__DEF_IINCHIP_BUS__ == __DEF_IINCHIP_SPI_MODE__)
		//SPI Initialize
	IINCHIP_SpiInit();

	IINCHIP_CSInit();
	IINCHIP_CSon();
	#endif  
}
